@@ -5,8 +5,9 @@ from construct import (Array, Byte, Embedded, Flag, Float32l, If, Int16sl, Int16
                        Switch, Pass, RepeatUntil, Bytes, LazyBound, IfThenElse)
 
 from mgz.enums import ObjectEnum, ObjectTypeEnum, ResourceEnum
-from mgz.util import Find, Version, find_version, find_save_version
+from mgz.util import Find, Version, find_version, find_save_version, find_type
 from mgz.header.unit_type import unit_type
+from mgz.header.de import de_string
 
 # pylint: disable=invalid-name
 
@@ -40,6 +41,16 @@ sprite_list = "sprite_list"/Struct(
     )))
 )
 
+particle = "particle"/Struct(
+    "type"/Int16ul,
+    Embedded(If(lambda ctx: ctx.type == 1, Struct(
+        "name"/de_string,
+        "x"/Float32l,
+        "y"/Float32l,
+        Bytes(26)
+    ))),
+)
+
 static = "static"/Struct(
     "object_type"/Int16ul,
     "sprite"/Int16ul,
@@ -61,7 +72,7 @@ static = "static"/Struct(
     "selected_group"/If(lambda ctx: find_version(ctx) == Version.AOK, Byte),
     ResourceEnum("resource_type"/Int16sl),
     "amount"/Float32l,
-    "worker_count"/Byte,
+    "worker_count"/IfThenElse(lambda ctx: find_save_version(ctx) == 12.36, Int32ul, Byte),
     "current_damage"/Byte,
     "damaged_lately_timer"/Byte,
     "under_attack"/Byte,
@@ -72,20 +83,33 @@ static = "static"/Struct(
     "de_static_unk1"/If(lambda ctx: find_version(ctx) == Version.DE, Bytes(19)),
     "has_sprite_list"/Byte,
     "sprite_list"/If(lambda ctx: ctx.has_sprite_list != 0, RepeatUntil(lambda x,lst,ctx: lst[-1].type == 0, sprite_list)),
-    "de_effect_block"/If(lambda ctx: find_version(ctx) == Version.DE, Struct(
+    "hd_extension"/If(lambda ctx: find_version(ctx) == Version.HD and find_save_version(ctx) > 12.36, Struct(
+        "flag"/Flag,
         Padding(4),
-        "has_effect"/Byte,
-        "effect"/If(lambda ctx: ctx.has_effect == 1, Struct(
-            Padding(3),
-            "length"/Int16ul,
-            "name"/Bytes(lambda ctx: ctx.length),
-            If(lambda ctx: ctx.length > 0, Padding(33))
-        )),
-        If(lambda ctx: ctx.effect is None or (ctx.effect and ctx.effect.length > 0), Byte),
-        Padding(4),
-        If(lambda ctx: find_save_version(ctx) >= 13.15, Padding(5)),
+        "has_array"/Int16ul,
+        "array"/If(lambda ctx: ctx.has_array, Struct(
+            "values"/RepeatUntil(lambda x,lst,ctx: lst[-1].type == 0, Struct(
+                "type"/Byte,
+                If(lambda ctx: ctx.type > 0, Bytes(16))
+            ))
+        ))
+    )),
+    "de_extension"/If(lambda ctx: find_version(ctx) == Version.DE, Struct(
+        "particles"/Array(5, particle),
+        If(lambda ctx: find_save_version(ctx) >= 13.15, Bytes(5)),
         If(lambda ctx: find_save_version(ctx) >= 13.17, Bytes(2)),
-        If(lambda ctx: find_save_version(ctx) >= 13.34, Bytes(12))
+        If(lambda ctx: find_save_version(ctx) >= 13.34, Struct(
+            Bytes(2),
+            de_string,
+            de_string,
+            Bytes(2)
+        )),
+        # not the right way to do this, needs improvement
+        If(lambda ctx: find_save_version(ctx) >= 20.16, Struct(
+            "peek"/Peek(Bytes(6)),
+            If(lambda ctx: find_type(ctx) == 10 and ctx.peek[0] == 0 and ctx.peek[0:2] != b"\x00\x0b", Bytes(1)),
+            If(lambda ctx: find_type(ctx) == 20 and ctx.peek[4] == 0 and ctx.peek[4:6] != b"\x00\x0b", Bytes(1)),
+        ))
     ))
 )
 
@@ -103,10 +127,11 @@ path_data = "path_data"/Struct(
     "waypoint"/Int32ul,
     "state"/Int32sl,
     "range"/Float32l,
-    "target_id"/Int32ul,
+    "target_id"/Int32sl,
     "pause_time"/Float32l,
     "continue_counter"/Int32ul,
-    "flags"/Int32ul
+    "flags"/Int32ul,
+    "hd"/If(lambda ctx: find_version(ctx) == Version.HD, Int32ul)
 )
 
 vector = Struct(
@@ -125,6 +150,7 @@ base_moving = "base_moving"/Struct(
     Embedded(animated),
     "trail_remainder"/Int32ul,
     "velocity"/vector,
+    "de_move_byte"/If(lambda ctx: find_save_version(ctx) >= 20.16, Byte),
     "angle"/Float32l,
     "turn_towards_time"/Int32ul,
     "turn_timer"/Int32ul,
@@ -154,7 +180,8 @@ base_moving = "base_moving"/Struct(
 
 moving = "moving"/Struct(
     Embedded(base_moving),
-    "de"/If(lambda ctx: find_version(ctx) == Version.DE, Bytes(17))
+    "hd_moving"/If(lambda ctx: find_version(ctx) == Version.HD, Bytes(1)),
+    "de_moving"/If(lambda ctx: find_version(ctx) == Version.DE, Bytes(17))
 )
 
 move_to = "move_to"/Struct(
@@ -186,7 +213,7 @@ attack = "attack"/Struct(
 unit_action = Struct(
     "type"/Int16ul,
     "data"/If(lambda ctx: ctx.type > 0, Struct(
-        "state"/IfThenElse(lambda ctx: find_version(ctx) in [Version.AOK, Version.AOC], Byte, Int32ul),
+        "state"/IfThenElse(lambda ctx: find_version(ctx) in [Version.AOK, Version.AOC, Version.HD], Byte, Int32ul),
         "target_object_pointer"/Int32sl,
         "target_object_pointer2"/Int32sl,
         "target_object_id"/Int32sl,
@@ -213,6 +240,7 @@ action_list = "action_list"/Struct(
 
 action = "action"/Struct(
     Embedded(base_moving),
+    "hd_action"/If(lambda ctx: find_version(ctx) == Version.HD, Bytes(3)),
     "waiting"/Byte,
     "command_flag"/Byte,
     "selected_group_info"/If(lambda ctx: find_version(ctx) != Version.AOK, Int16ul),
@@ -358,7 +386,8 @@ combat = "combat"/Struct(
     "has_ai"/Int32ul,
     "ai"/If(lambda ctx: ctx.has_ai > 0, unit_ai),
     "peek"/Peek(Bytes(5)), # TODO: figure out the right way to do this part
-    "de_position"/If(lambda ctx: ctx.peek != b'\x00\xff\xff\xff\xff', Struct(
+    "hd_position"/If(lambda ctx: find_version(ctx) == Version.HD and ctx.peek != b'\x00\xff\xff\xff\xff', Bytes(4)),
+    "de_position"/If(lambda ctx: find_version(ctx) == Version.DE and ctx.peek != b'\x00\xff\xff\xff\xff', Struct(
         "position"/vector,
         "flag"/Byte,
     )),
@@ -372,7 +401,8 @@ combat = "combat"/Struct(
     "berserker_timer"/Float32l,
     "num_builders"/Byte,
     "num_healers"/If(lambda ctx: find_version(ctx) != Version.AOK, Byte),
-    "de_unknown"/If(lambda ctx: find_save_version(ctx) >= 20.06, Int32ul)
+    "de_unknown"/If(lambda ctx: find_save_version(ctx) >= 20.06, Int32ul),
+    "de_unknown2"/If(lambda ctx: find_save_version(ctx) >= 25.01, Int32ul)
 )
 
 production_queue = "production_queue"/Struct(
@@ -395,7 +425,7 @@ building = "building"/Struct(
     "desolid_flag"/Byte,
     "pending_order"/Int32ul,
     "linked_owner"/Int32sl,
-    "linked_children"/Array(lambda ctx: 3 if find_version(ctx) == Version.DE else 4, Int32sl),
+    "linked_children"/Array(lambda ctx: 3 if find_version(ctx) in (Version.DE, Version.HD) else 4, Int32sl),
     "captured_unit_count"/Byte,
     "extra_actions"/action_list,
     "research_actions"/If(lambda ctx: find_version(ctx) != Version.DE, action_list),
@@ -413,7 +443,8 @@ building = "building"/Struct(
     "terrain_type"/Byte,
     "semi_asleep"/Byte,
     "snow_flag"/If(lambda ctx: find_version(ctx) != Version.AOK, Byte),
-    "de_flag_unk"/If(lambda ctx: find_version(ctx) == Version.DE, Byte)
+    "de_flag_unk"/If(lambda ctx: find_version(ctx) == Version.DE, Byte),
+    "de_unk_2"/If(lambda ctx: find_save_version(ctx) >= 20.16, Int16ul)
 )
 
 # Objects that exist on the map at the start of the recorded game
@@ -431,7 +462,7 @@ existing_object = "objects"/Struct(
         70: combat,
         80: building,
         90: static
-    }, default=Pass)),
+    }, default=Pass))
 )
 
 # Default values for objects, nothing of real interest
